	INCLUDE EFR32MG12.inc		; CPU register definitions
	INCLUDE terminal.inc		; terminal function definitions	
	INCLUDE	defines.inc	
	EXPORT	main
	IMPORT	GPIO_setup		; import names from other code files
	IMPORT	TIMER0_setup
	IMPORT	LETIMER_setup	
	IMPORT	TRNG_setup	
	
;-------------------------------------	; CODE segment in flash (ROM) 
	AREA    |.text|, CODE, READONLY
main	PROC				; main user code
	bl	Terminal_setup		; setup TeraTerm communication
	bl	GPIO_setup		; configure port pins
	bl	TRNG_setup
	bl	LETIMER_setup
	bl	TIMER0_setup

loop					
	bl	Get_TRN			; R1 = TRN
	ldr	R1, =LETIMER0		; set LETIMER period
	str	R0, [R1, #LETIMERn_COMP0]
	mov	R0, #5
	str	R0, [R1, #LETIMERn_CMD]	; start LETIMER
	
	wfi				; wait for pause end
	OORW	RED_LED, GPIO_PD_DOUT, GPIO ; turn on red LED
	PUTW	1, TIMERn_CMD, TIMER0	; start TIMER0
	
	wfi				; wait for BUTTON0 press
	ANDW	~RED_LED, GPIO_PD_DOUT, GPIO ; turn off red LED
	
	GETW	TIMERn_CC1_CCV, TIMER0	; R1 = reaction count
	cbz	R1, L1
	mov	R2, #1000		; compute reaction time as
	mul	R1, R2			;   R1*1000ms/18554
	mov	R2, #18554
	udiv	R1, R2			; R1 = time in ms

	ldr	R0, =prompt1		; print reaction time
	bl	WriteString
	mov	R0, R1
	bl	WriteInt
	ldr	R0, =prompt2
	bl	WriteString
	b	loop
	
L1	ldr	R0, =prompt3		; print "You are too slow..."
	bl	WriteString
	b	loop			; repeat periodic task		
	ENDP

;---------------------------------------------------
Get_TRN	PROC				; get TRN in R1
	push	{R1-R3}			; preserve used regs	
	GETW	TRNGn_FIFO, TRNG0	; R1 = TR number

	mov	R2, #MIN_TRN		; min TRN
	mov	R3, #MAX_TRN
	sub	R3, R2			; max TRN	
	udiv	R0, R1, R3
	mls	R1, R0, R3, R1 	
	add	R0, R1, R2		; R0 = TRN in the range [R2..R3]	
	pop	{R1-R3}
	bx	LR
	ENDP
		
;-------------------------------------
prompt1	DCB	"Your reaction time is ", 0	; allocate constant strings here
prompt2	DCB	"ms", 13,10, 0
prompt3	DCB	"You are too slow...", 13, 10, 0

	ALIGN
	END